/* Sx cpu micro architecture simulator

   5 December 2001
   modify from s2m sim
   Computer Architecture 1997, 1998
   microprogram version 		13 Feb 2005

   sx micro						10 July 2006
   sx0  minimum version			11 July 2006

   integrate to noss			15 Aug 2006

   P. Chongstitvatana
   Department of Computer Engineering
   Chulalongkorn University
*/

#include "sx.h"

#define TIMEOUT  500		// time quanta

extern int DP;				// load.c
extern int clock, dt;
extern int intflag;			// interrupt flag

void initchip(void){
	clock = 0;
	ninst = 0;
	runflag = 1;
	TS = 0;
	NX = 0;
	FF = 0;
	FP = SSBASE;
	SP = SSBASE;
	PC = 1;
	Z = 0;
	S = 0;
	mpc = 0;
}

void decode(int code, int *op, int *arg){
	*op = code & 255;
	*arg = code >> 8;
}

PRIVATE void popts(void){
	TS = M[SP];
	SP--;
}

// static alloc memory of size a
PRIVATE int xalloc(int a){
	int b;
	b = DP;
	if( DP + a >= MAXDS )
		error("xalloc: out of memory");
	DP += a;
	return b;
}
/*
PRIVATE void report(void){
  float cpi;
  if( ninst == 0 )
	printf("0 instruction\n");
  else {
	cpi = ((float) clock)/ninst;
	printf("%d instructions, %d cycles, CPI %4.2f\n",
		ninst,clock,cpi);
  }
}
*/

PRIVATE void trace(void){
	int op, arg;
//	if(ninst > 500) exit(0);
	decode(M[PC],&op,&arg);
	if(op == icCall)
		printf("call %s\n",searchref(tyFUN,arg));
	else
		prCode(stdout,M[PC],PC);
	printf(" ");
}

void yield(void){
	if(intflag && (clock-dt) > TIMEOUT)
		noss(time_out);
}

// processor execution
void eval(int ref){
	PC = ref;
	while(runflag ) {
//		trace();
		run3();		// execute one instruction
		ninst++;
		yield();
	}
}

void trap(int op, int arg){		// special functions
	switch(op) {
	case icEnd : noss(stopped);	break;
	case icArray: TS = xalloc(TS); break;
	case icSys:
		switch(arg){
		case 1: 				// print ts
			printf("%d",TS);
			popts();
			break;
		case 2 : 				// printc ts
			printf("%c",TS);
			popts();
			break;
		case 13: runflag = 0; break;
		case 20: intflag = 1; break;
		case 21: intflag = 0; break;
		case 22: noss(blocked); break;
		default:
			error("unknown syscall");
		}
		break;
	default:
		error("unknown trap");
	}
}

/*
int main(int argc, char *argv[]){
  if( argc < 2 ) {
	printf("usage : sx objfile\n");
	exit(0);
  }
  loadobj(argv[1]);
  initchip();
  run();
  return 0;
}
*/
